#include "c_interface.hpp"

/* ********************************************************** */
/*                      CONVERSION FUNCTION                   */
/* ********************************************************** */

static inline std::string stringXtoStd(const XString _str)
{
   char * const temp = new char[_str.len+1]();
   memset(temp, '\0', _str.len+1);
   memcpy(temp, _str.str, _str.len);
   std::string _retstr(temp);
   size_t d = _retstr.find_first_not_of(' ');
   size_t f = _retstr.find_last_not_of (' ');

   delete[] temp;
   return (_retstr.substr(d, f-d+1));
}

static inline XString stringXtoStd(const std::string& _str) // Non testée
{
   XString _retstr = {new char[_str.size()](), _str.size()};
   memcpy(_retstr.str, _str.c_str(), _str.size());
   return (_retstr);
}

static inline Duration durationXtoXMLIO(const XDuration * const _xdur) // Non testée
{
   Duration __dur= { _xdur->year, _xdur->month , _xdur->day,
                     _xdur->hour, _xdur->minute, _xdur->second };
   return (__dur);
}

/* ********************************************************** */
/*                      HANDLE INTERFACE                      */
/* ********************************************************** */

void xios_handle_create_(XHandle * const _ret, const XDType * const _dtype, const XString _id)
{
   // Si le handle n'est pas initialisé à 0, on ne fait aucun traitement.
   if (!isNullHandle(*_ret)) return;
   std::string __id = stringXtoStd(_id);
   switch(*_dtype)
   {
      /* EAXIS, EDOMAIN, EFIELD, EFILE, EGRID, GAXIS, GDOMAIN, GFIELD, GFILE, GGRID*/
      case (ECONTEXT):
         // Si le context n'existe pas, on retourne un handle vide/nul.
         if (!Context::HasContext(__id)) { *_ret = NULLHANDLE; return ; }
         _ret->data_type = *_dtype;
         _ret->data_ptr  = Context::GetContext(__id);
         return ;

      default :
         *_ret = NULLHANDLE;
         return ;
   };
}

/* ********************************************************** */
/*                      XML INTERFACE                         */
/* ********************************************************** */

void xios_xml_parse_file_(const XString _filename)
{
   std::string   __filename = stringXtoStd(_filename);
   std::ifstream __istr( __filename.c_str() , std::ifstream::in );

   // On commence la lecture du flux de donnée xml.
   XMLNode node = XMLNode::CreateNode(__istr, Context::GetRootName());

   // On parse le fichier xml noeud par noeud
   // (ie on construit dynamiquement notre arbre d'objets).
   XMLParser::Parse(node);
}

void xios_xml_parse_string_(const XString _xmlcontent)
{
   std::string        __xmlcontent = stringXtoStd(_xmlcontent);
   std::istringstream __istr(__xmlcontent);

   // On commence la lecture du flux de donnée xml.
   XMLNode node = XMLNode::CreateNode(__istr, Context::GetRootName());

   // On parse le fichier xml noeud par noeud
   // (ie on construit dynamiquement notre arbre d'objets).
   XMLParser::Parse(node);
}

/* ********************************************************** */
/*                      DATA TREATMENT INTERFACE              */
/* ********************************************************** */

static std::vector<DataTreatment *> AllDataTreatment;

static void deleteAllDataTreatment(void)
{
   std::vector<DataTreatment *>::iterator it;
   for (it  = AllDataTreatment.begin();
        it != AllDataTreatment.end(); it++)
      if (*it != NULL) delete (*it);
}

void xios_dtreatment_new_(XHandle * const _dt, const XHandle * const _hd)
{
   static bool called = false;

   // Si le handle de traitement n'est pas initialisé à 0, on ne fait aucun traitement.
   if (!isNullHandle(*_dt) || (_hd->data_type != ECONTEXT)) return;
   Context       * const __ctxt = (Context*) _hd->data_ptr;
   DataTreatment * const __dtrt =__ctxt->setDataTreatment<DataTreatment>();

   if (__dtrt != NULL)
   {
      AllDataTreatment.push_back(__dtrt);
      _dt->data_type = DTREATMENT;
      _dt->data_ptr  = __dtrt;
      if (!called)
      {
         atexit (&deleteAllDataTreatment);
         called = true;
      }
   }
}

void xios_dtreatment_create_files_and_headers_(const XHandle * const _hd, const XFileType * const _ft)
{
   if (isNullHandle(*_hd) || (_hd->data_type != DTREATMENT)) return;
   DataTreatment * const __dt = (DataTreatment*)_hd->data_ptr;
   switch(*_ft)
   {
      case (NETCDF4):
         __dt->createDataOutput<NetCDF4DataOutput>();
         return;
      // Autres formats de fichiers si disponibles...
      default:
         return;
   }
}

/* ********************************************************** */
/*                      CONTEXT INTERFACE                     */
/* ********************************************************** */

void xios_context_set_current_(const XHandle * const _ctxt, const bool * const _wswap)
{
   bool __wswap = (_wswap == NULL) ? false : *_wswap;
   if (isNullHandle(*_ctxt) || (_ctxt->data_type != ECONTEXT)) return;
   Context* const __ctxt = (Context*) _ctxt->data_ptr;
   Context::SetCurrentContext(__ctxt->getId(), __wswap) ;
}

void xios_context_get_current_ (XHandle * _ctxt)
{
   // Si le handle de traitement n'est pas initialisé à 0, on ne fait aucun traitement.
   if ((Context::GetAllListObject().size() == 0) ||
         !isNullHandle(*_ctxt)) return;
   _ctxt->data_type = ECONTEXT;
   _ctxt->data_ptr  = Context::GetCurrentContext();
}

void xios_context_get_calendar_(const XHandle * const _ctxt, XHandle * _cal)
{
   if (!isNullHandle(*_cal)  || isNullHandle(*_ctxt) ||
      (_ctxt->data_type != ECONTEXT)) return;
   Context* const __ctxt = (Context*) _ctxt->data_ptr;
   AbstractCalendar * const __cal  = __ctxt->getCalendar();
   if (__cal != NULL)
   {
      _cal->data_type = CALENDAR;
      _cal->data_ptr  = __cal;
   }
}

/* ********************************************************** */
/*                      CALENDAR INTERFACE                    */
/* ********************************************************** */

void xios_calendar_set_timestep_(const XHandle * const _cal, const XDuration * const _dur)
{
   if (isNullHandle(*_cal) || (_cal->data_type != CALENDAR)) return;
   const Duration __dur = durationXtoXMLIO(_dur);
   AbstractCalendar * const __cal = (AbstractCalendar*) _cal->data_ptr;
   __cal->setTimeStep(__dur);
}

#undef isNullHandle
